syntax = "proto3";

package node;

message Node {
  string RemoteSocket = 1;   // 远程socket tcp://*:5555
  string LocalSocket = 2; // 本地socket
  string NodeName = 3;    // 节点名称
  string Domain = 4;      // 作用域
  string Category = 5;    // 类别
  string Key = 6;         // 节点key
}
