syntax = "proto3";

import "more_than_inference/protos/node.proto";
import "more_than_inference/protos/data.proto";

package pipeline;

message Pipeline {
  string Key = 1;
  repeated node.Node StackList = 2;
  data.Data Data= 3;
}
