syntax = "proto3";

import "more_than_inference/protos/node.proto";

package resource;

message Resource {
  int32 CPU = 1;
  int32 RAM = 2;
  int32 GPU = 3;
  int32 VRAM = 4;
  node.Node node = 5;
}

message Apply {
  bool Accept = 1;
  string Env = 2;
}