syntax = "proto3";

package data;

message Data {
    bytes Body = 1;         // 数据体
    string Serializer = 2;  // 序列化器
    uint32 Seq = 3;         // 内容序列index，兼容SSE/WebSocket的流式数据传输
    bool Ended = 4;         // 是否完成
}